#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright 2010 Johan Dobbelstein
# 
#               DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
#                        Version 2, December 2004
# 
# Copyright (C) 2004 
# Sam Hocevar 14 rue de Plaisance, 75014 Paris, France
# Everyone is permitted to copy and distribute verbatim or modified copies of
# this license document, and changing it is allowed as long as the name is
# changed.
# 
#               DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE 
#   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
# 
#   0. You just DO WHAT THE FUCK YOU WANT TO.


"""
Remotely manage DotClear2 posts from the CLI.

Using a small configuration file (default to dotclear.cfg), list, create,
modify or delete blog posts. If the configuration file set a text editor name,
it will be used to edit the content of the post.

For more information on MovableType API, see
http://www.sixapart.com/developers/xmlrpc/ 
"""

import xmlrpclib
import xml.dom.minidom
import ConfigParser, datetime, getpass
import sys, getopt
import tempfile, os

CFG_DEFAULT = "dotclear.cfg"
BALISE = "<!-- %s EXCERPT SEPARATOR %s -->" % (":"*10, ":"*10)
XHTML_TEMPLATE = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
    <title>XML-RPC post preview</title>
</head>
<body>
%s
</body>
</html>
"""

class MyBlog():
    def __init__(self, config_file):
        """ Initialise thanks to a configuration file 
            and create a xmlrpclib.Server. """
        config = ConfigParser.ConfigParser()
        config.read(config_file)
        self.blogid = config.get('Blog', "blogid")
        self.url = config.get('Blog', "url")
        if (self.url[0:5] != "https"):
            print >> sys.stderr, "WARNING: URL is not secured, use https instead of http://"
            print >> sys.stderr, "         data transmission (username/password) is unsecured\n"
        self.username = config.get('Blog', "username")
        try :
            self.password = config.get('Blog', "password")
        except ConfigParser.NoOptionError:
            self.password = getpass.getpass("Please enter your password for %s:\n" % self.url)
        try :
            self.editor = config.get('Common', 'editor')
        except ConfigParser.NoSectionError, ConfigParser.NoOptionError:
            self.editor = None
        try :
            self.server = xmlrpclib.Server( self.url )
        except xmlrpclib.Fault, fault:
            print >> sys.stderr, "Fail to connect to server"
            print >> sys.stderr, "Error code = ", fault.faultCode
            print >> sys.stderr, "Error message = ", fault.faultString

    def _selectCategories(self):
        " return an array of struct with categoryId and isPrimary fields "
        categories = self.server.mt.getCategoryList(
            self.blogid, self.username, self.password
        )
        for i,c in enumerate(categories):
            print i,c['description']
        r = None
        while (r == None) :
            try :
                choice = int(raw_input("categorie number ?\n"))
                if (choice < len(categories)) and ( choice >= 0 ):
                    r = [ {'categoryId': categories[choice]['categoryId'] } ]
                else :
                    print >> sys.stderr, "Out of bound !"
            except ValueError:
                print >> sys.stderr, "Not a integer !"
        return r

    def _setCategorie(self, postid):
        categories = self._selectCategories()
        try :
            self.server.mt.setPostCategories( 
                postid, 
                self.username, self.password, 
                categories
            )
        except xmlrpclib.Fault, fault:
            print >> sys.stderr, "Fail to set category"
            print >> sys.stderr, "Error code = ", fault.faultCode
            print >> sys.stderr, "Error message = ", fault.faultString


    def _externalEditor(self,prev_content):
        """ 
        Dump existing content to a temporary file and open it for modification
        with self.editor.
        Check XML syntax too.
        """
        f, filename = tempfile.mkstemp()
        fd = os.fdopen(f, 'wb')
        fd.write(prev_content.encode("utf-8"))
        fd.close()

        # Looping on "data" until we get well-formatted XML
        valid_xml = False
        while not valid_xml :
            try :
                raw_input("Launching external text editor. Press enter to continue")
                os.system(self.editor + " " +filename)

                f = open(filename, 'rb')
                text = f.read()
                f.close()
                doc = xml.dom.minidom.parseString(text)
                valid_xml = True
            except xml.parsers.expat.ExpatError as err:
                print >>sys.stderr, "\nERROR: Not valid XML"
                print >>sys.stderr, err
                print >>sys.stderr, "Please, try again :p\n"
            except Exception as err :
                print >>sys.stderr, "\nERROR: Unknown when parsing XML"
                print >>sys.stderr, type(err)
                print >>sys.stderr, err
                print >>sys.stderr, "Please, try again :p\n"
        text = doc.getElementsByTagName("body")[0].toxml("utf-8")
        text = text.replace("<body>", "").replace("</body>", "")

        os.unlink(filename)
        return text

    def listLastPosts(self, number):
        """
        List the last `number` posts
        """
        try :
             entries = self.server.mt.getRecentPostTitles( 
                 self.blogid, self.username,
                 self.password, number
             )
        except xmlrpclib.Fault, fault:
            print >> sys.stderr, "Fail to get list of recent posts"
            print >> sys.stderr, "Error code = ", fault.faultCode
            print >> sys.stderr, "Error message = ", fault.faultString
        for e in entries :
            category = e['categories'][0] if (e.has_key("categories")) else 'nil'
            print "%s :: %s in cat. %s (tags: %s)" % (e['postid'], e['title'], 
                                              category, e['mt_keywords'])
        print

    def showPost(self, id):
        """ 
        Print the raw content (xHTML) of post `id`
        """
        post = self._extractPost(id)
        date = datetime.datetime.strptime( 
            post['dateCreated'].value,
            "%Y%m%dT%H:%M:%S"
        )
        print "-" * 79
        print "[ %s ] === %s ===" % (post['postid'], post['title'] )
        if (post.has_key('categories')):
            print "By %s on %s - %s" % (post['userid'], date.ctime(), post['categories'][0])
        else :
            print "By %s on %s" % (post['userid'], date.ctime())
        print "( tags: %s )" % post['mt_keywords']
        print
        print post['formatted_text']
        print 
        print "Link: " + post['link']
        print "permaLink: " + post['permaLink']
        print "-" * 79

    def _updateField(self, prompt, string):
        r = raw_input(prompt + " [default:" + string + "]\n")
        if r == "" :
            return string
        else :
            return r

    def newPost(self):
        """ Create a post from scratch. """
        print
        content = {}
        content['title'] = raw_input("Title?\n")
        content['mt_keywords'] = raw_input("Tags? (comma separated list)\n")
        # Categories are not included in the struct "content"
        # see _setCategorie()
        if self.editor : 
            prev_data = "<p>Insert here excerpt</p>\n\t" + BALISE + "\n\t<p>Insert here post main content</p>"
            data = self._externalEditor(XHTML_TEMPLATE % prev_data )
            content['mt_excerpt'], content['description'] = data.split(BALISE)
        else :
            content['mt_excerpt'] = raw_input("Excerpt? (beware of xHTML tags !)\n")
            content['description'] = raw_input("Main content? (beware of xHTML tags !)\n")
        content['mt_allow_comments'] = raw_input("Allow comments?[Y|n] ") != "n"
        content['mt_allow_pings'] = raw_input("Allow pings?[Y|n] ") != "n"
        publish = raw_input("Publish ?[Y|n]") != "n"
        try :
            postid = self.server.metaWeblog.newPost(
                self.blogid, self.username, self.password,
                content, publish
            )
        except xmlrpclib.Fault, fault:
            print >> sys.stderr, "Fail to post the new entry :("
            print >> sys.stderr, "Error code = ", fault.faultCode
            print >> sys.stderr, "Error message = ", fault.faultString
        else :
            self._setCategorie(postid)
            print "New post created with ID =", postid

    def editPost(self, id):
        """ Modify post given its ID. """
        old_data = self._extractPost(id)
        print 
        content = {} 
        content['title'] = self._updateField("Title?\n", old_data['title']) 
        content['mt_keywords'] = self._updateField( "Tags? (comma separated lists)?\n", old_data['mt_keywords'])
        # Categories are not included in the struct "content"
        # see _setCategorie()
        if self.editor : 
            prev_data = old_data['formatted_text']
            data = self._externalEditor(XHTML_TEMPLATE % prev_data)
            # if no content before balise, Python
            # s.split thorw an exception
            if data.find(BALISE) == 1 :
                content['description'] = data[len(BALISES):]
            else :
                content['mt_excerpt'], content['description'] = data.split(BALISE)
        else :
            content['mt_excerpt'] = self._updateField( "Excerpt? (beware of xHTML tags !)\n", old_data['mt_excerpt'])
            content['description'] = self._updateField( "Main content? (beware of xHTML tags !)\n", old_data['description'])
        content['mt_allow_comments'] = raw_input("Allow comments?[Y|n] ") != "n"
        content['mt_allow_pings'] = raw_input("Allow pings?[Y|n] ") != "n"
        publish = raw_input("Publish ?[Y|n]") != "n"
        try :
            self.server.metaWeblog.editPost(
                id, self.username, self.password,
                content, publish
            )
            self._setCategorie(id)
        except xmlrpclib.Fault, fault:
            print >> sys.stderr, "Fail to edit entry :("
            print >> sys.stderr, "Error code = ", fault.faultCode
            print >> sys.stderr, "Error message = ", fault.faultString

    def deletePost(self, postid):
        #N.B. the first value ('appkey') is ignored
        # the fourth value is a boolean named 'Publish' and I cannot figure it
        # out its use
        try :
            self.server.blogger.deletePost(
                "appkey", postid, self.username, self.password, True)
        except xmlrpclib.Fault, fault:
            print >> sys.stderr, "Failed to delete post", postid
            print >> sys.stderr, "Error code =", fault.faultCode
            print >> sys.stderr, "Error message =", fault.faultString

    def _extractPost(self, id):
        """ Returne post content. """
        try :
            data =  self.server.metaWeblog.getPost(
                id, self.username, self.password
            ) 
            data['formatted_text'] = "\t" + data['mt_excerpt'] + "\n\t" + BALISE + "\n\t" + data['description']
        except xmlrpclib.Fault, fault:
            print >> sys.stderr, "Fail to get post content"
            print >> sys.stderr, "Error code = ", fault.faultCode
            print >> sys.stderr, "Error message = ", fault.faultString
        return data

    def extractPost(self, id):
        """ Print post content to STDOUT. """
        print self._extractPost(id)['formatted_text']

if __name__ == '__main__':
    def usage():
        print >> sys.stderr, "Usage: %s [options]" % sys.argv[0]
        print >> sys.stderr, """
        -h, --help              Display this message
        -c, --config=filename   Read configuration from filename
                                (default to dotclear.cfg)
        -l, --list=n            List the last n posts
        -s, --show=id           Show post given its id
        -n, --new               Create a new entry
        -d, --delete=id         Delete an entry
        -e, --edit=id           Edit post given its id
        -x, --extract=id        Print to stdout the content of post id
        """
        sys.exit(1)

    try :
        opts, args = getopt.gnu_getopt(sys.argv[1:],
           "hc:l:s:ne:d:x:",
           ["help", "config=", "list=", "show=", "new", "edit=", "delete=", "extract="])
    except getopt.GetoptError, err:
        # print help information and exit:
        print >> sys.stderr, str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)

    # First scan: display help OR get configuration file
    cfg_filename = CFG_DEFAULT
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
        elif o in ("-c", "--config"):
            cfg_filename = a
        elif o in ("-l", "--list", "-s", "--show", "-n", "--new", "-e",
                   "--edit", "-d", "--delete", "-x", "--extract"):
            continue
        else :
            print >> sys.stderr, "Error: unhandled option"
            usage()

    # Second scan: action !
    myblog = MyBlog(cfg_filename)
    did_something = False
    for o, a in opts:
        if o in ("-l", "--list"):
            myblog.listLastPosts(a)
            did_something = True
        elif o in ("-s", "--show"):
            myblog.showPost(str(a))
            did_something = True
        elif o in ("-n", "--new"):
            myblog.newPost()
            did_something = True
        elif o in ("-e", "--edit"):
            myblog.editPost(a)
            did_something = True
        elif o in ("-d", "--delete"):
            myblog.deletePost(a)
            did_something = True
        elif o in ("-x", "--extract"):
            myblog.extractPost(a)
            did_something = True
    if not did_something :
        usage()
